@@ -743,16 +743,25 @@ getConv <- function(x) {
     } else x[["convergence"]]
 }
 
+getMsg <- function(x) {
+    if (!is.null(x[["msg"]])) {
+        x[["msg"]]
+    } else if (!is.null(x[["message"]])) {
+        x[["message"]]
+    } else ""
+}
+
 .optinfo <- function(opt, lme4conv=NULL)
     list(optimizer = attr(opt, "optimizer"),
          control   = attr(opt, "control"),
          derivs    = attr(opt, "derivs"),
          conv      = list(opt = getConv(opt), lme4 = lme4conv),
          feval     = if (is.null(opt$feval)) NA else opt$feval,
+         message   = getMsg(opt),
          warnings  = attr(opt, "warnings"),
          val       = opt$par)
 
-##' Potentically needed in more than one place, be sure to keep consistency!
+##' Potentially needed in more than one place, be sure to keep consistency!
 ##' hack (NB families have weird names) from @aosmith16; then corrected
 isNBfamily <- function(familyString)
     grepl("^Negative ?Binomial", familyString, ignore.case=TRUE)
@@ -895,41 +904,13 @@ checkArgs <- function(type,...) {
 ## if #3 is true *and* the user is doing something tricky with nested functions,
 ## this may fail ...
 
-## from http://stackoverflow.com/questions/14945274/determine-whether-evaluation-of-an-argument-will-fail-due-to-non-existence
-missDataFun <- function(d) {
-    ## test here
-    ff <- sys.frames()
-    ex <- substitute(d)
-    ii <- rev(seq_along(ff))
-    foundAnon <- FALSE
-    for(i in ii) {
-        ## trying to find where the original symbol is defined ...
-        ex <- eval(substitute(substitute(x, env=sys.frames()[[n]]),
-                              env = list(x = ex, n=i)))
-        if (is.symbol(ex) && grepl("^\\.\\.[0-9]+$",safeDeparse(ex))) {
-            ## testing for the dreaded "..1", "..2" pattern; this means
-            ## we are stuck in an anonymous function somewhere ...
-            ## won't be able to see whether data exist or not,
-            ## but we should not flag them as missing -- definitely
-            ## causes false positives
-            ## might be better to check against quote(..1), quote(..2),
-            ## ... but ugh ...
-            foundAnon <- TRUE
-            break
-        }
-    }
-    return(!foundAnon && is.symbol(ex) && !exists(deparse(ex)))
-}
-
 ## try to diagnose missing/bad data
 checkFormulaData <- function(formula, data, checkLHS=TRUE,
                              checkData=TRUE, debug=FALSE) {
-    nonexist.data <- missDataFun(data)
-    wd <- tryCatch(eval(data), error = identity)
-    if (wrong.data <- inherits(wd,"simpleError")) {
-        wrong.data.msg <- wd$message
+    wd <- tryCatch(force(data), error = identity)
+    if (bad.data <- inherits(wd,"error")) {
+        bad.data.msg <- wd$message
     }
-    bad.data <- nonexist.data || wrong.data
 
     ## data not found (this *should* only happen with garbage input,
     ## OR when strings used as formulae -> drop1/update/etc.)
@@ -945,11 +926,7 @@ checkFormulaData <- function(formula, data, checkLHS=TRUE,
                  "try specifying 'formula' as a formula rather ",
                  "than a string in the original model",call.=FALSE)
         } else {
-            if (nonexist.data) {
-                stop("'data' not found, and some variables missing from formula environment",call.=FALSE)
-            } else {
-                stop("bad 'data': ",wrong.data.msg)
-            }
+            stop("bad 'data': ", bad.data.msg, call. = FALSE)
         }
     } else {
         denv <- ## The data as environment
@@ -1175,7 +1152,10 @@ nloptwrap <- local({
         with(res, list(par   = solution,
                        fval  = objective,
                        feval = iterations,
-                       conv  = if(status > 0) 0 else status,
+                       ## ?nloptr: "integer value with the status of the optimization (0 is success)"
+                       ## most status>0 are fine (e.g. 4 "stopped because xtol_rel was reached"
+                       ## but status 5 is "ran out of evaluations"
+                       conv  = if (status<0 || status==5) status else 0,
                        message = message))
     }
 })
@@ -1293,3 +1273,5 @@ isSingular <- function(x, tol = 1e-4) {
     theta <- getME(x, "theta")
     any(theta[lwr==0] < tol)
 }
+
+lme4_testlevel <- function() if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1
