@@ -1,22 +1,25 @@
 ## ----preliminaries,include=FALSE,cache=FALSE,message=FALSE----------
 options(width=70, show.signif.stars=FALSE,
-             str=strOptions(strict.width="cut"),
-             prompt = "R> ", continue = "+  ",
-             useFancyQuotes = FALSE)
+        str=strOptions(strict.width="cut"),
+        ## prefer empty continuation for reader's cut'n'paste:
+        continue = "   ", #JSS: prompt = "R> ", continue = "+  ",
+        useFancyQuotes = FALSE)
 library("knitr")
 library("lme4")
-library("ggplot2")
-theme_set(theme_bw())
+library("ggplot2")# Keeping default theme, nicer "on line":
+#JSS theme_set(theme_bw())
 library("grid")
-zmargin <- theme(panel.margin=unit(0,"lines"))
+zmargin <- theme(panel.spacing=unit(0,"lines"))
 library("lattice")
-opts_chunk$set(engine='R',dev='pdf',fig.width=9,prompt=TRUE,
-               fig.height=5.5,cache=TRUE,tidy=FALSE,comment=NA)
+library("minqa")
+opts_chunk$set(engine='R',dev='pdf', fig.width=9, fig.height=5.5,
+               prompt=TRUE, cache=TRUE, tidy=FALSE, comment=NA)
+render_sweave()
 
 ## ----sleep----------------------------------------------------------
 str(sleepstudy)
 
-## ----sleepPlot,echo=FALSE,fig.scap="Reaction time versus days by subject",fig.cap="Average reaction time versus days of sleep deprivation by subject.  Subjects ordered by increasing slope of subject-specific linear regressions.",fig.align='center',fig.pos='tb'----
+## ----sleepPlot, echo=FALSE, fig.scap="Reaction time versus days by subject", fig.cap="Average reaction time versus days of sleep deprivation by subject.  Subjects ordered (from left to right starting on the top row) by increasing slope of subject-specific linear regressions.", fig.align='center', fig.pos='tb'----
 ## BMB: seemed more pleasing to arrange by increasing slope rather than
 ## intercept ...
 xyplot(Reaction ~ Days | Subject, sleepstudy, aspect = "xy",
@@ -66,12 +69,13 @@ set.seed(2)
 
 ## ----sanityCheck, include=FALSE-------------------------------------
 ## alternative formulation of Zi (eq:Zi)
-rBind(Ji[1,] %x% Xi[1,],
-      Ji[2,] %x% Xi[2,],
-      Ji[3,] %x% Xi[3,],
-      Ji[4,] %x% Xi[4,],
-      Ji[5,] %x% Xi[5,],
-      Ji[6,] %x% Xi[6,])
+rBind(
+  Ji[1,] %x% Xi[1,],
+  Ji[2,] %x% Xi[2,],
+  Ji[3,] %x% Xi[3,],
+  Ji[4,] %x% Xi[4,],
+  Ji[5,] %x% Xi[5,],
+  Ji[6,] %x% Xi[6,])
 
 ## ----nc, echo=FALSE-------------------------------------------------
 nc <- 3
@@ -129,9 +133,9 @@ Lambdati
 #  beta[] <- as.vector(solve(RXtRX, XtWy - crossprod(RZX, cu)))
 #  u[] <- as.vector(solve(L, solve(L, cu - RZX %*% beta,
 #                                  system = "Lt"), system = "Pt"))
-#  b[] <- as.vector(crossprod(Lambdat, u))
 
 ## ----PLSupdateResp, eval = FALSE------------------------------------
+#  b[] <- as.vector(crossprod(Lambdat, u))
 #  mu[] <- as.vector(crossprod(Zt, b) + X %*% beta + offset)
 #  wtres <- sqrtW * (y - mu)
 
@@ -278,228 +282,65 @@ densityplot(pf)
 splom(pf)
 
 ## ----modularSimulationFormula---------------------------------------
-form <- respVar ~ 1 + (explVar1 | groupFac1) + (explVar2 | groupFac2)
+form <- respVar ~ 1 + (explVar1 + explVar2 | groupFac)
 
-## ----modularSimulationInit------------------------------------------
+## ----dataTemplate---------------------------------------------------
 set.seed(1)
-nGrps <- 50
-
-## ----modularSimulationExpl------------------------------------------
-explVar <- data.frame(explVar1 = rnorm(nGrps^2),
-                      explVar2 = rnorm(nGrps^2))
-
-## ----modularSimulationGroup-----------------------------------------
-groupFac <- expand.grid(groupFac1 = as.factor(1:nGrps),
-                        groupFac2 = as.factor(1:nGrps))
-
-## ----modularSimulationInterSlope------------------------------------
-randomIntercept <- expand.grid(randomIntercept1 = rnorm(nGrps),
-                               randomIntercept2 = rnorm(nGrps))
-rnmdSlope <- expand.grid(randomSlope1 = rnorm(nGrps),
-                         randomSlope2 = rnorm(nGrps)) - randomIntercept
-
-## ----modularSimulationPredictor-------------------------------------
-linearPredictor <- apply(randomIntercept + rnmdSlope * explVar, 1, sum)
-residError <- rnorm(nGrps^2)
-respVar <- residError + linearPredictor
-
-## ----modularSimulationDataFrame-------------------------------------
-dat <- data.frame(respVar, explVar, groupFac)
+dat <- mkDataTemplate(form,
+  nGrps = 500,
+  nPerGrp = 20,
+  rfunc = rnorm)
 head(dat)
 
-## ----homoLmer, echo=FALSE-------------------------------------------
-homoLmer <- function(formula, data, use.mkMerMod = FALSE) {
-    mc <- match.call()
+## ----parsTemplate---------------------------------------------------
+(pars <- mkParsTemplate(form, dat))
 
-                                        # parse the formula and make
-                                        # two copies
-    lfHetero <- lfHomo <- lFormula(formula = formula, data = data, REML = FALSE)
-                                        # get pRE (number of random
-                                        # effects coefficients per
-                                        # term)...
-    if(length(pRE <- unique(sapply(cnms <- lfHomo$reTrms$cnms, length))) > 1L) {
-        stop("each random effects term must have the same number\n",
-             "of model matrix columns for a homogeneous structure")
-    }
-                                        # and p (number of fixed
-                                        # effects coefficients)
-    p <- ncol(lfHomo$X)
-    nth <- choose(pRE + 1, 2)
-    n_trms <- length(cnms)
-                                        # modify one copy (lfHomo) of the
-                                        # parsed formula
-    lfHomo$reTrms <- within(lfHomo$reTrms, {
-                                        # take only the first three
-                                        # (of six) elements of the
-                                        # covariance parameters,
-                                        # theta, because these will be
-                                        # repeated (to get common
-                                        # covariance structure across
-                                        # random effects terms)
-        theta <- theta[1:nth]
-                                        # match the lower bounds to
-                                        # the new theta
-        lower <- lower[1:nth]
-                                        # adjust the mapping from the
-                                        # new theta to the elements of
-                                        # the relative covariance
-                                        # factor, Lambda
-        Lind <- rep(1:nth, length = length(lfHomo$reTrms$Lambdat@x))
-    })
+## ----simCorr--------------------------------------------------------
+vc <- matrix(c(1.0, 0.5, 0.5,
+  0.5, 1.0, 0.0,
+  0.5, 0.0, 1.0), 3, 3)
 
-                                        # fit the modified model
-    devf <- do.call(mkLmerDevfun, lfHomo)
-    opt <- optimizeLmer(devf)
+## ----vcTheta--------------------------------------------------------
+pars$theta[] <- Vv_to_Cv(mlist2vec(vc))
 
-                                        # how to process the output?
-    if(!use.mkMerMod) {
-        return(list(lf = lfHomo, devf = devf, opt = opt))
-    } else {
+## ----varCorrStructure-----------------------------------------------
+dat$respVar <- simulate(form,
+  newdata = dat,
+  newparams = pars,
+  family = "gaussian")[[1]]
 
-                                        # distinguish between theta
-                                        # with only the unique
-                                        # elements (thetaUnique) and
-                                        # theta with the unique
-                                        # elements repeated over each
-                                        # term (thetaExpand)
-        thetaUnique <- opt$par
-        thetaExpand <- rep(opt$par, n_trms)
-                                        # construct deviance a
-                                        # function for the heterogeneous
-                                        # model...
-        devfHetero <- do.call(mkLmerDevfun, lfHetero)
-                                        # ...and install the expanded
-                                        # covariance parameters from
-                                        # the optimized homogeneous
-                                        # model, into the opt object
-                                        # and the environment of the
-                                        # heterogeneous model deviance
-                                        # function...
-        devfHetero(opt$par <- thetaExpand)
-                                        # ...and creating the final
-                                        # merMod object with identical
-                                        # random effects parameters
-                                        # for both random effects
-                                        # terms
-        mod <- mkMerMod(environment(devfHetero), opt, lfHetero$reTrms, lfHetero$fr)
+## ----graphSims------------------------------------------------------
+formLm <- form
+formLm[[3]] <- findbars(form)[[1]]
+print(formLm)
+cor(t(sapply(lmList(formLm, dat), coef)))
 
-                                        # extend the glmerMod class
-                                        # and return
-        modList <- setNames(lapply(slotNames(mod), slot, object = mod), slotNames(mod))
-        makeFakeCall <- function() return(match.call()) # hack to keep
-                                                    # "new" from
-                                                    # trying to
-                                                    # evaluate the
-                                                    # call...
-        modList$call <- makeFakeCall()
-        modCallList <- c(list(Class = "homoLmerMod"),
-                         list(thetaUnique = thetaUnique),
-                         modList)
-        modOut <- do.call("new", modCallList)
-                                        # ... complete the "call" hack
-                                        # by putting the real matched
-                                        # call back before returning
-        modOut@call <- mc
-        return(modOut)
-    }
+## ----phiToTheta, cache = FALSE--------------------------------------
+phiToTheta <- function(phi) {
+    theta5 <- -(phi[2]*phi[3])/phi[4]
+    c(phi[1:4], theta5, phi[5])
 }
 
-## ----homoLmerArguments, eval=FALSE----------------------------------
-#  homoLmer <- function(formula, data, use.mkMerMod = FALSE) {}
+## ----compute deviance function modular, cache = FALSE---------------
+lf <- lFormula(formula = form, data = dat, REML = TRUE)
+devf <- do.call(mkLmerDevfun, lf)
 
-## ----homoLmerMatchCallAndParseFormula, eval=FALSE-------------------
-#  mc <- match.call()
-#  lfHetero <- lfHomo <- lFormula(formula = formula, data = data, REML = FALSE)
+## ----wrapper modular, cache = FALSE---------------------------------
+devfWrap <- function(phi) devf(phiToTheta(phi))
 
-## ----homoLmerCheck, eval=FALSE--------------------------------------
-#  if(length(pRE <- unique(sapply(cnms <- lfHomo$reTrms$cnms, length))) > 1L) {
-#      stop("each random effects term must have the same number\n",
-#           "of model matrix columns for a homogeneous structure")
-#  }
+## ----opt modular, cache = FALSE-------------------------------------
+opt <- bobyqa(par = lf$reTrms$theta[-5],
+  fn = devfWrap,
+  lower = lf$reTrms$lower[-5])
 
-## ----homoLmerDimensions, eval=FALSE---------------------------------
-#  p <- ncol(lfHomo$X)
-#  nth <- choose(pRE + 1, 2)
-#  n_trms <- length(cnms)
-
-## ----homoLmerAdjust, eval=FALSE-------------------------------------
-#  lfHomo$reTrms <- within(lfHomo$reTrms, {
-#      theta <- theta[1:nth]
-#      lower <- lower[1:nth]
-#      Lind <- rep(1:nth, length = length(lfHomo$reTrms$Lambdat@x))
-#  })
-
-## ----homoLmerFitModifiedModel, eval=FALSE---------------------------
-#  devf <- do.call(mkLmerDevfun, lfHomo)
-#  opt <- optimizeLmer(devf)
-
-## ----homoLmerReturnRaw, eval=FALSE----------------------------------
-#  if(!use.mkMerMod) {
-#      return(list(lf = lfHomo, devf = devf, opt = opt))
-#  }
-
-## ----homoLmerReExpandTheta, eval=FALSE------------------------------
-#  th <- rep(opt$par, n_trms)
-
-## ----unmodDevfun, eval=FALSE----------------------------------------
-#  devf0 <- do.call(mkLmerDevfun, lfHetero)
-
-## ----installDevfun, eval=FALSE--------------------------------------
-#  devf0(opt$par <- th)
-
-## ----extendMerMod---------------------------------------------------
-setClass("homoLmerMod", representation(thetaUnique = "numeric"),
-         contains = "lmerMod")
-
-## ----redefineOutputModuleMethods------------------------------------
-logLik.homoLmerMod <- function(object, ...) {
-    ll <- lme4:::logLik.merMod(object, ...)
-    attr(ll, "df") <- length(object@beta) +
-        length(object@thetaUnique) +
-            object@devcomp[["dims"]][["useSc"]]
-    return(ll)
-}
-refitML.homoLmerMod <- function(object, newresp, ...) {
-    if(!isREML(object) && missing(newresp)) return(object)
-    stop("can't refit homoLmerMod objects yet")
-}
-
-## ----homoLmerFit----------------------------------------------------
-mod <- homoLmer(form, dat, use.mkMerMod = TRUE)
-summary(mod)
-
-## ----homoParse------------------------------------------------------
-parsedFormula <- lFormula(formula = Reaction ~ Days + (Days | Subject),
-                          data = sleepstudy)
-
-## ----homoWithin-----------------------------------------------------
-parsedFormula$reTrms <- within(parsedFormula$reTrms, {
-    q <- nrow(Lambdat)
-    Lind <- rep(1, q)
-    Lambdat <- sparseMatrix(1:q, 1:q, x = Lind)
-    theta <- 1
-})
-
-## ----homoFitting----------------------------------------------------
-devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
-optimizerOutput <- optimizeLmer(devianceFunction)
-
-## ----homoMkMerMod---------------------------------------------------
-mkMerMod(   rho = environment(devianceFunction),
-            opt = optimizerOutput,
-         reTrms = parsedFormula$reTrms,
-             fr = parsedFormula$fr)
-
-## ----homoModifiedProcessing-----------------------------------------
-with(environment(devianceFunction), {
-    n <- length(resp$y)
-    p <- length(pp$beta0)
-    pwrss <- resp$wrss() + pp$sqrL(1)
-    sig <- sqrt(pwrss/(n-p))
-    c(Residual = sig, Subject = sig*pp$theta)
-})
+## ----varCorr modular, cache = FALSE---------------------------------
+vcEst <- vec2mlist(Cv_to_Vv(phiToTheta(opt$par)))[[1]]
+dimnames(vcEst) <- rep(lf$reTrms$cnms, 2)
+round(vcEst, 2)
+vc
 
 ## ----simulateSplineData,message=FALSE-------------------------------
+library("gamm4")
 library("splines")
 set.seed(1)
 n <- 100
